function H = HSH_basis(phi, theta, N)
% Compute the hemispherical harmonics (HSH) basis functions.
% 
% Input:
% phi: nv x 1 azimuth angle in [-pi, pi]
% theta: nv x 1 elevation angle in [0, pi/2]
% N: maximum order
% 
% Output:
% H: nv x (N+1)^2 matrix of the HSH basis functions
% 
% Reference:
% A. Giri, G. P. T. Choi, L. Kumar, 
% "Open and closed anatomical surface description via hemispherical area-preserving map."
% Signal Processing, 180, 107867, 2021.

H = zeros(size(theta,1),(N+1)^2);
P_cos = 2*cos(theta)-1; 
for n = 0:N
    Pn = legendre(n,P_cos)';
    for m = -n:n
        K = sqrt(((2*n+1)*factorial(n-abs(m)))/(2*pi*factorial(n+abs(m))));
        if m < 0
            H(:,n^2+n+m+1) = (-1)^abs(m)*sqrt(2)*K.*sin(abs(m).*phi).*Pn(:,abs(m)+1);
        elseif m > 0
            H(:,n^2+n+m+1) = (-1)^abs(m)*sqrt(2)*K.*cos(m*phi).*Pn(:,abs(m)+1);
        else
            H(:,n^2+n+1) = K.*Pn(:,1);
        end
    end
end
