function map = hemispherical_area_preserving_map(v,f)
% Compute a hemispherical area-preserving map of a simply-connected open surface
%
% Input:
% v: nv x 3 vertex coordinates of a simply-connected open triangle mesh
% f: nf x 3 triangulations of a simply-connected open triangle mesh
% 
% Output:
% map: nv x 3 vertex coordinates of the open hemispherical area-preserving parameterization
% 
% Reference:
% A. Giri, G. P. T. Choi, L. Kumar, 
% "Open and closed anatomical surface description via hemispherical area-preserving map."
% Signal Processing, 180, 107867, 2021.
%
% Copyright (c) 2020-2021, Gary P. T. Choi
% https://math.mit.edu/~ptchoi

%% Step 1: Initial flattening map
% based on disk conformal map (Choi and Lui, J. Sci. Comput. 2015)
disk = disk_conformal_map(v,f);

%% Step 2: Compute a hemispherical area-preserving map using optimal mass transport
% based on OMT (Nadeem et al., TVCG 2017)
bd = meshboundaries(f);
uv_bdy = disk(bd{1},:);

% prepare the source and target measure for the OMT computation
va = vertex_area(f,v)/3;
va_uv = vertex_area(f,disk)/3;
% normalize the target area such that sum(va) = int_\Omega sigma
va = va/sum(va)*sum(va_uv.*(4./(1+disk(:,1).^2+disk(:,2).^2).^2));
sigma = @(xy) (4./(1+xy(:,1).^2+xy(:,2).^2).^2); % source measure, with the conformal factor of the stereogrpahic projection
delta = va; % target measure
option.eps = 1e-3;
option.max_iter = 15; 

% run the OMT map
[pd,~] = discrete_optimal_transport(uv_bdy,f,disk,sigma,delta,[],option);
uv_final = compute_centroid(uv_bdy,pd);
    
% inverse stereographic projection
z = complex(uv_final(:,1), uv_final(:,2));
map = [2*real(z)./(1+abs(z).^2), 2*imag(z)./(1+abs(z).^2), -(-1+abs(z).^2)./(1+abs(z).^2)];

end
