% A fast method for computing hemispherical area-preserving map of simply-connected triangle meshes
%
% Main program:
% map = hemispherical_area_preserving_map(v,f)
%
% Input:
% v: nv x 3 vertex coordinates of a simply-connected open triangle mesh
% f: nf x 3 triangulations of a simply-connected open triangle mesh
% 
% Output:
% map: nv x 3 vertex coordinates of the hemispherical area-preserving parameterization
% 
% Reference:
% A. Giri, G. P. T. Choi, L. Kumar, 
% "Open and closed anatomical surface description via hemispherical area-preserving map."
% Signal Processing, 180, 107867, 2021.
%
% Copyright (c) 2020-2021, Gary P. T. Choi
% https://math.mit.edu/~ptchoi

addpath(genpath('code'));

%% Load a mesh
load('vol_OAS2_0103_MR1Scan1_open.mat');

plot_mesh(v,f);
view([-125 10]); 
title('Input surface')

%% Compute a hemispherical area-preserving map
map = hemispherical_area_preserving_map(v,f);

plot_mesh(map,f); 
title('Hemispherical parameterization')
view([-25 20]); 

%% Reconstruction using hemispherical harmonics
N = 15; % maximum order
theta = acos(map(:,3)./(sqrt(map(:,1).^2+map(:,2).^2+map(:,3).^2))); 
phi = atan2(map(:,2),map(:,1)); 
H = HSH_basis(phi, theta, N);

mult = (H'*H)\H';
coeff = mult*v;
v_reconstruct = H*coeff;

plot_mesh(v_reconstruct,f); 
title(['HSH reconstruction with N = ', num2str(N)]);
view([-125 10]); 